package com.isu.gaswellwatch.modbus.data.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.ResourceUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 15:42
 */
public abstract class AbstractPersistenceHandler implements PersistenceHandler {

    @Resource
    protected JdbcTemplate jdbcTemplate;
    @Resource
    protected SnowflakeConfig snowflakeConfig;
    @Resource(name = "stringRedisTemplate")
    protected RedisTemplate redisTemplate;

    protected void createTable(String filePath, String tableName, Long deviceId) {
        String createTableSQL = this.getResource(filePath);
        createTableSQL = StringUtils.replace(createTableSQL, "$TableName$", tableName);
        createTableSQL = StringUtils.replace(createTableSQL, "$DeviceId$", String.valueOf(deviceId));
        this.jdbcTemplate.execute(createTableSQL);
    }

    protected Map<String, Object> getLastRow(String tableName) {
        return this.jdbcTemplate.queryForList("SELECT * FROM " + tableName + " ORDER BY ID DESC LIMIT 1")
                .stream()
                .findFirst()
                .orElse(null);
    }

    protected void setValue(PreparedStatement ps, Map<String, Object> row, int index, String key, int sqlType) throws SQLException {
        String value = MapUtil.getStr(row, key);
        if (Objects.isNull(value)) {
            ps.setNull(index, sqlType);
        } else {
            ps.setObject(index, value);
        }
    }

    protected String getResource(String classPath) {
        try {
            File file = ResourceUtils.getFile("classpath:" + classPath);
            return IOUtils.toString(new FileInputStream(file), StandardCharsets.UTF_8);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

}
