package com.isu.gaswellwatch.utils;

import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/25 16:07
 */
public class ReverseComplementCodeUtils {

    /**
     * 原码转反码<br/>
     * 正数的原码，反码，补码都一样（三码合一）, 0的反码，补码都是0<br/>
     * 负数的反码=它的原码符号不变，其他位取反（0 => 1 1 => 0）
     *
     * @param source 二进制原码
     * @return
     */
    public static String binOriginalToReverse(String source) {
        if (source.startsWith("0")) {
            return source;
        } else {
            StringBuilder sbf = new StringBuilder(source.length());
            sbf.append("1");
            String f_str = source.substring(1);
            for (int i = 0; i < f_str.length(); i++) {
                String s_str = String.valueOf(f_str.charAt(i));
                if (s_str.equals("0")) {
                    sbf.append("1");
                } else if (s_str.equals("1")) {
                    sbf.append("0");
                }
            }
            return sbf.toString();
        }
    }

    /**
     * 反码转补码, 默认补1<br/>
     * 负数的补码 = 它的反码 + 1，负数的反码 = 负数的补码 - 1
     *
     * @param reverse 二进制反码字符串
     * @return
     */
    public static String binReverseToComplement(String reverse) {
        return binReverseToComplement(reverse, "1");
    }

    /**
     * 反码转补码
     *
     * @param reverse    二进制反码字符串
     * @param complement 二进制补数字符串
     * @return
     */
    public static String binReverseToComplement(String reverse, String complement) {
        if (reverse.startsWith("0")) {
            return reverse;
        }
        StringBuilder sb = new StringBuilder(reverse.length());
        int x = 0;
        int y = 0;
        int pre = 0;//进位
        int sum = 0;//存储进位和另两个位的和
        while (reverse.length() != complement.length()) {//将两个二进制的数位数补齐,在短的前面添0
            if (reverse.length() > complement.length()) {
                complement = "0" + complement;
            } else {
                reverse = "0" + reverse;
            }
        }
        for (int i = reverse.length() - 1; i >= 0; i--) {
            x = reverse.charAt(i) - '0';
            y = complement.charAt(i) - '0';
            sum = x + y + pre;//从低位做加法
            if (sum >= 2) {
                pre = 1;//进位
                sb.append(sum - 2);
            } else {
                pre = 0;
                sb.append(sum);
            }
        }
        if (pre == 1) {
            sb.append("1");
        }
        // 翻转返回
        return sb.reverse().toString();
    }

    /**
     * 补码转十进制
     *
     * @param
     * @return
     */
    public static String binComplementToDec(String complement) {
        String binNumber = complement.substring(1);
        String firstNumber = complement.substring(0, 1);
        if (Objects.equals(firstNumber, "0")) {
            return "" + Long.valueOf(binNumber, 2);
        } else if (Objects.equals(firstNumber, "1")) {
            return "-" + Long.valueOf(binNumber, 2);
        }
        return null;
    }

    public static void main(String[] args) {
//        String hexNumber = "F7"; // 16进制负数表示，等同于-9
        String hexNumber = "F6"; // 16进制负数表示，等同于-10
//        String hexNumber = "E6"; // 16进制负数表示，等同于-26
        int decimalNumber = Integer.parseInt(hexNumber, 16);

        String binaryNumber = Integer.toBinaryString(decimalNumber);
        System.out.println("有符号16进制数：" + hexNumber + "，对应的2进制数为：" + binaryNumber);

        binaryNumber = binOriginalToReverse(binaryNumber);
        System.out.println("有符号16进制数：" + hexNumber + "，对应反码2进制为：" + binaryNumber + "，值为：" + Integer.parseInt(binaryNumber, 2));

        binaryNumber = binReverseToComplement(binaryNumber, "1");
        System.out.println("有符号16进制数：" + hexNumber + "，对应反码的补码为：" + binaryNumber + "，值为：" + Integer.parseInt(binaryNumber, 2));

        System.out.println("有符号16进制数：" + hexNumber + "，对应的10进制数为：" + binComplementToDec(binaryNumber));
    }
}
